%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%     Optimal Growth model with irreversible investment	      %
%                                                             %
%           PEA Galerkin method (Markov Chain case)           %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
global kmin ksup XX kt;
global nstate nbk ncoef XX XT PI;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Algorithm Parameters                        %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nbk=10;				% Degree of polynomials
nodes=30;			% # of Nodes
nstate=5;			% # of possible states for the income process
ncoef=nbk+1;		% # of coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                    Structural Parameters                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
delta	= 0.1;
beta	= 0.95;
alpha	= 0.3;
sigma	= 1.5;
ysk	    =(1-beta*(1-delta))/(alpha*beta);
ksy	    = 1/ysk;
ys		= ksy^(alpha/(1-alpha));
ks		= ys^(1/alpha);
is		= delta*ks;
cs		= ys-is;
%
% Markov Chain: technology shock
%
rho	    = 0.8;
se		= 0.075;
ma		= 0;
[agrid,wmat]=hernodes(nstate);
agrid   = agrid*sqrt(2)*se; 
PI      = transprob(agrid,wmat,0,rho,se); 
at      = agrid+ma;
%
% grid for the capital stock
%
kmin	= log(1);
ksup	= log(7);
rk		= rcheb(nodes);					% roots
kt		= itransfo(rk,kmin,ksup);		% grid
XX		= cheb(rk,[0:nbk]);				% Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                     Initial Conditions                      %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a0=[   
-0.32518704    -0.22798091    -0.15021991    -0.12387075    -0.23608269
-0.61535041    -0.64879442    -0.70031744    -0.84056683    -1.23998020
-0.02892469    -0.03369096    -0.05570433    -0.15063263    -0.45685868
-0.00565378    -0.00941495    -0.02618621    -0.09526710    -0.32031660
-0.00296230    -0.00547789    -0.01692339    -0.06102182    -0.21319594
-0.00145337    -0.00274142    -0.00988611    -0.03461623    -0.13635186
-0.00065225    -0.00115070    -0.00536657    -0.01776343    -0.09027963
-0.00025245    -0.00038302    -0.00291504    -0.00838555    -0.06431768
-0.00004845    -0.00001360    -0.00162292    -0.00365588    -0.04689460
 0.00001150     0.00013398    -0.00088759    -0.00124601    -0.03294968
-0.00003248     0.00013595    -0.00055706    -0.00013116    -0.02163670
];
 a0=a0(:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          Main loop                          %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param=[alpha beta delta sigma]';
th=fcsolve('residuals',a0,[],param,at);
disp('Final rule : ');
th		= reshape(th,ncoef,nstate);
disp(th')

lt		= length(th);
nb		= 100;
kt		= [kmin:(ksup-kmin)/(nb-1):ksup]';
rk		= transfo(kt,kmin,ksup);
XX		= cheb(rk(:),[0:nbk]);
kt		= exp(kt);
kp      = zeros(nb,nstate);
it      = zeros(nb,nstate);
ct      = zeros(nb,nstate);
mu      = zeros(nb,nstate);
for i=1:nstate;
   Upsilon  = exp(XX*th(:,i));
   y        = exp(at(i))*kt.^alpha;
   it(:,i)  = max(y-Upsilon.^(-1/sigma),0);
   ct(:,i)  = y-it(:,i);
   kp(:,i)  = it(:,i)+(1-delta)*kt;
   mu(:,i)  = ct(:,i).^(-sigma)-Upsilon;
end;


subplot(221);plot(kt,kp);
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(kt,ct);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(223);plot(kt,it);
xlabel('k_t','fontname','times','fontsize',12)
title('Investment','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(224);plot(kt,mu);
xlabel('k_t','fontname','times','fontsize',12)
title('Lagrange multiplier','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause;
print -depsc2 irpeamc.eps
close